-- The addon namespace.
local addon, ns = ...

-- Define the console class and put it in the addon namespace.
local Console = {}
local ConsoleMetatable = { __index = Console }
ns.Console = Console

-- Constructor.
function Console:new(manager)
	local self = {}
	setmetatable(self, ConsoleMetatable)
	
	self.Manager = manager
	
	-- Slash command handler
	function self.SlashCommandHandler(msg, editbox)
		local command, rest = msg:match("^(%S*)%s*(.-)$");
		command = strlower(command)
		rest = strlower(rest)
		if command == "" or command == "help" then
			Console.Write("Usage: /shieldmaid <command> or /sm <command>")
			Console.Write("Commands:")
			Console.Write("help")
			Console.Write("reset")
			Console.Write("lock")
			Console.Write("unlock")
			Console.Write("secondaryBarrier true/false")
			Console.Write("size <pixels>")
			Console.Write("scale <number>")
			Console.Write("margin <pixels>")
			Console.Write("hiddenOutOfCombat true/false")
			Console.Write("showFrames true/false")
			Console.Write("showGlow true/false")
			Console.Write("showCooldown true/false")
			Console.Write("truncatedNumbers true/false")
		elseif command == "reset" then
			self.Manager:Reset()	
			Console.Write("Configuration reset")
		elseif command == "lock" then
			self.Manager:Lock()
			Console.Write("Frames locked")
		elseif command == "unlock" then
			self.Manager:Unlock()
			Console.Write("Frames unlocked")
		elseif (command == "secondarybarrier") or (command == "secondary") or (command == "barrier") or (command == "sb") or (command == "sbarrier") or (command == "secondaryb") then
			if rest == "true" then
				self.Manager:SetSecondaryBarrier(true)
				Console.Write("Estimated Ignore pain value will be displayed while the buff is active.")
			elseif rest == "false" then
				self.Manager:SetSecondaryBarrier(false)
				Console.Write("Estimated Ignore pain value will no longer be displayed while the buff is active.")
			else	
				if (ShieldMaidConfig.secondaryBarrier) then
					Console.Write("Estimated value of Ignore Pain is currently shown on the icon while Ignore Pain is active. (true)")
				else
					Console.Write("Estimated value of Ignore Pain is currently hidden on the icon while Ignore Pain is active. (false)")
				end
			end
		elseif command == "size" then
			local size = tonumber(rest)
			if size then
				self.Manager:SetSize(size)
				Console.Write("Size set to "..size)
			else
				Console.Write("Size is currently set to: "..ShieldMaidConfig.size)
			end
		elseif command == "scale" then
			local scale = tonumber(rest)
			if scale then
				self.Manager:SetScale(scale)
				Console.Write("Scale set to "..scale)
			else
				Console.Write("Scale is currently set to: "..ShieldMaidConfig.scale)
			end
		elseif command == "margin" then
			local margin = tonumber(rest)
			if margin then
				self.Manager:SetMargin(margin)
				Console.Write("Margin set to "..margin)
			else
				Console.Write("Margin is currently set to: "..ShieldMaidConfig.margin)
			end
		elseif (command == "hiddenoutofcombat") or (command == "hiddenooc") or (command == "hidden") or (command == "combat") or (command == "hooc") or (command == "ooc") then
			if rest == "true" then
				self.Manager:SetHiddenOutOfCombat(true)
				Console.Write("Icons will be hidden out of combat.")
			elseif rest == "false" then
				self.Manager:SetHiddenOutOfCombat(false)
				Console.Write("Icons will be visible out of combat.")
			else
				if (ShieldMaidConfig.hiddenOutOfCombat) then
					Console.Write("Icons are currently hidden while not in combat. (true)")
				else
					Console.Write("Icons are currently visible while not in combat. (false)")
				end
			end
		elseif (command == "showframes") or (command == "sf") or (command == "frames") or (command == "sframes") or (command == "showf") then
			if rest == "true" then
				self.Manager:SetShowFrames(true)
				Console.Write("Frames will be visible.")
			elseif rest == "false" then
				self.Manager:SetShowFrames(false)
				Console.Write("Frames will be hidden.")
			else
				if (ShieldMaidConfig.showFrames) then
					Console.Write("Icon frames are currently visible. (true)")
				else
					Console.Write("Icon frames are currently hidden. (false)")
				end
			end
		elseif (command == "showglow") or (command == "glow") or (command == "sg") or (command == "sglow") or (command == "showg") then
			if rest == "true" then
				self.Manager:SetShowGlow(true)
				Console.Write("Glow will be visible.")
			elseif rest == "false" then
				self.Manager:SetShowGlow(false)
				Console.Write("Glow will be hidden.")
			else
				if (ShieldMaidConfig.showGlow) then
					Console.Write("Icon glows are currently visible. (true)")
				else
					Console.Write("Icon glows are currently hidden. (false)")
				end
			end

		elseif (command == "showcooldown") or (command == "cooldown") or (command == "sc") or (command == "showc") or (command == "scooldown") then
			if rest == "true" then
				self.Manager:SetShowCooldown(true)
				Console.Write("The cooldown 'clock' will be visible.")
			elseif rest == "false" then
				self.Manager:SetShowCooldown(false)
				Console.Write("The cooldown 'clock' will be hidden.")
			else
				if (ShieldMaidConfig.showCooldown) then
					Console.Write("Icon cooldowns are currently visible. (true)")
				else
					Console.Write("Icon cooldowns are currently hidden. (false)")
				end
			end
	
			elseif (command == "truncatednumbers") or (command == "truncate") or (command == "numbers") or (command == "tn") or (command == "truncated") then
			if rest == "true" then
				self.Manager:SetTruncatedNumbers(true)
				Console.Write("Numbers will now be truncated.")
			elseif rest == "false" then
				self.Manager:SetTruncatedNumbers(false)
				Console.Write("Numbers will no longer be truncated.")
			else
				if (ShieldMaidConfig.truncatedNumbers) then
					Console.Write("Numbers are currently truncated. (true)")
				else
					Console.Write("Numbers are not currently truncated. (false)")
				end
			end
			
		else
			Console.Write("Unknown command: "..command)
		end
	end
	
	return self
end

-- Utility function for writing to the chat window.
function Console.Write(text)
	print("|c00C79C6EShield Maid:|r "..text)
end

-- Register slash command handler.
function Console:RegisterSlashCommands()
	SLASH_SHIELDMAID1, SLASH_SHIELDMAID2 = "/shieldmaid", "/sm"
	SlashCmdList["SHIELDMAID"] = self.SlashCommandHandler
end